@@ -26,7 +26,7 @@ atoms_table['ImaginaryUnit'] = junicode
 
 pretty_expr_map = {sym.I: junicode, sym.Heaviside: 'u'}
 
-# Words to format in Roman font for LaTeX expressions. 
+# Words to format in Roman font for LaTeX expressions.
 
 functions = ('heaviside', 'diracdelta', 'conjugate', 'sqrt', 'exp',
              'log', 'log10', 'sin', 'cos', 'tan', 'cot' 'asin',
@@ -40,10 +40,10 @@ subscripts = ('in', 'out', 'ref', 'rms', 'load', 'source', 'avg',
               'cc', 'ee', 'dd', 'ss', 'ih', 'il', 'oh', 'ol',
               'typ', 'pkg', 'comp', 'step')
 
-greek_letter_names = ('alpha', 'beta', 'gamma', 'delta', 'eta', 'zeta',
-                      'theta', 'iota', 'kappa', 'lambda', 'mu', 'nu',
-                      'omicron', 'pi', 'rho', 'sigma', 'tau',
-                      'upsilon', 'chi', 'psi', 'omega')
+greek_letter_names = ('alpha', 'beta', 'gamma', 'delta', 'epislon',
+                      'zeta', 'eta', 'theta', 'iota', 'kappa', 'lambda',
+                      'mu', 'nu', 'xi', 'omicron', 'pi', 'rho', 'sigma', 'tau',
+                      'upsilon', 'phi', 'chi', 'psi', 'omega')
 
 words = greek_letter_names
 
@@ -55,7 +55,7 @@ try:
     matrix_inverse_method = 'DM-charpoly'
 except:
     matrix_inverse_method = 'ADJ'
-    
+
 matrix_inverse_fallback_method = 'ADJ'
 
 
