@@ -1,7 +1,7 @@
 """This module wraps SymPy functions and provides a few others.
 The underlying SymPy function is obtained with the sympy attribute.
 
-Copyright 2014--2021 Michael Hayes, UCECE
+Copyright 2014--2022 Michael Hayes, UCECE
 """
 
 from .units import u as uu
@@ -16,7 +16,7 @@ __all__ = ('conjugate', 'sqrt', 'exp', 'log', 'log10', 'sin', 'cos',
            'rect', 'sinc', 'sincn', 'sincu', 'psinc', 'tri', 'trap',
            'Sum', 'dtrect', 'dtsign', 'Piecewise', 'Eq', 'Ne',
            'Lt', 'Le', 'Gt', 'Ge', 'Derivative', 'Integral', 'Max', 'Min',
-           're', 'im')
+           're', 'im', 'MatMul', 'MatAdd')
 
 from .extrafunctions import UnitImpulse as UnitImpulse1
 from .extrafunctions import UnitStep as UnitStep1
@@ -37,17 +37,17 @@ class Function(object):
             # Handled AppliedUndef
             arg = sym.Function(arg)
         self.expr = arg
-    
+
     @property
     def sympy(self):
         """Return SymPy expression."""
-        
+
         return self.expr
 
     def __call__(self, *args, **kwargs):
 
         func = self.expr
-            
+
         e_args = list(map(expr, args))
 
         if func == sym.Piecewise:
@@ -56,16 +56,16 @@ class Function(object):
             # Integral(const, (tau, 0, t)) -> use t class
             # Integral(integrand, (tau, a, b)) -> use integrand class
             if e_args[0].is_constant:
-                cls = e_args[1][2].__class__                
+                cls = e_args[1][2].__class__
             else:
-                cls = e_args[0].__class__            
+                cls = e_args[0].__class__
         else:
             cls = e_args[0].__class__
-            
+
             # Handle cases like atan2(1, omega)
             if len(e_args) > 1:
                 if e_args[0].is_constant:
-                    cls = e_args[1].__class__                
+                    cls = e_args[1].__class__
 
         try:
             if cls.is_discrete_time_domain or cls.is_discrete_fourier_domain:
@@ -77,11 +77,11 @@ class Function(object):
                         break
         except:
             pass
-                
+
         result = func(*delcapify(e_args), **kwargs)
 
-        result = cls(result)        
-            
+        result = cls(result)
+
         if (self.expr in (sym.sin, sym.cos, sym.tan, sym.cot, sym.exp,
                           sym.sinh, sym.cosh, sym.tanh, sym.coth)
             and e_args[0].is_phase):
@@ -96,21 +96,21 @@ class Function(object):
             elif isinstance(e_args[1], tuple) and isinstance(e_args[1][0], Expr):
                 result.units = e_args[0].units * e_args[1][0].units
         elif self.expr is sym.DiracDelta and isinstance(e_args[0], Expr):
-                result.units = 1 / e_args[0].units                
+                result.units = 1 / e_args[0].units
 
         return result
 
     def pdb(self):
         import pdb; pdb.set_trace()
         return self
-    
-    
+
+
 class Log10(Function):
 
     # TODO, figure out how to print as log10(x) rather than
     # the expansion log(x) / log(10).  This will require
     # deferment of the expansion.
-    
+
     def __call__(self, arg):
         return super(Log10, self).__call__(arg, 10)
 
@@ -118,15 +118,15 @@ class Log10(Function):
 class SincnFunction(Function):
     """Normalized sinc function :math:`\sin(\pi x)/(\pi x)`."""
 
-    
+
 class SincuFunction(Function):
     """Unnormalized sinc function :math:`\sin(x)/(x)`."""
 
-    
-class PsincFunction(Function):
-    """Periodic sinc function :math:`\sin(M * x)/(M * sin(x))`."""        
 
-    
+class PsincFunction(Function):
+    """Periodic sinc function :math:`\sin(M * x)/(M * sin(x))`."""
+
+
 conjugate = Function(sym.conjugate)
 
 sqrt = Function(sym.sqrt)
@@ -219,6 +219,10 @@ re = Function(sym.re)
 
 im = Function(sym.im)
 
+MatMul = Function(sym.MatMul)
+
+MatAdd = Function(sym.MatAdd)
+
 ui = unitimpulse = UnitImpulse = Function(UnitImpulse1)
 
 us = unitstep = UnitStep = Function(UnitStep1)
