@@ -1,7 +1,7 @@
 """This module provides support for the unilateral inverse Laplace
 transform.
 
-Copyright 2021 Michael Hayes, UCECE
+Copyright 2021--2022 Michael Hayes, UCECE
 
 """
 
@@ -123,7 +123,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
 
         if delay != 0:
             # This will be caught and trigger expansion of the expression.
-            raise ValueError('Unhandled delay %s' % delay)
+            self.error('Unhandled delay %s' % delay)
 
         cresult = Zero
 
@@ -298,9 +298,122 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         result = inverse_laplace_transform(expr, s, t)
 
         if result.has(sym.InverseLaplaceTransform):
-            raise ValueError('SymPy cannot find inverse Laplace transform of %s' % expr)
+            self.error('SymPy does not know either')
         return result
 
+    def tline_end(self, expr, s, t):
+        """Attempt to find response at end of unterminated lossless
+        transmission line."""
+
+        # Look for expression of form 1 / (a * cosh(s * T) + b * sinh(s * T))
+
+        if not expr.is_Pow or expr.args[1] != -1:
+            self.error('Not 1 / X')
+
+        denom = expr.args[0]
+
+        arg = None
+        a = Zero
+        b = Zero
+
+        for term in denom.expand().as_ordered_terms():
+
+            const, e = factor_const(term, s)
+            if e.is_Function and e.func == sym.cosh:
+                if arg is None:
+                    arg = e.args[0]
+                elif arg != e.args[0]:
+                    self.error('Mismatch cosh arg')
+                a += const
+            elif e.is_Function and e.func == sym.sinh:
+                if arg is None:
+                    arg = e.args[0]
+                elif arg != e.args[0]:
+                    self.error('Mismatch sinh arg')
+                b += const
+            else:
+                self.error('Term does not include cosh or sinh')
+
+        T = arg / s
+        if T.has(s):
+            self.error('Frequency dependent delay')
+
+        m = self.dummy_var(expr, 'm', level=0, real=True)
+        g = (b - a) / (b + a)
+        if g == 0:
+            h = 1 / (b + a) * sym.DiracDelta(t - T)
+        else:
+            h = 1 / (b + a) * sym.Sum(g**m * sym.DiracDelta(t - (2 * m + 1) * T), (m, 0, sym.oo))
+        return h
+
+    def tline_start(self, expr, s, t):
+        """Attempt to find response at start of unterminated lossless
+        transmission line."""
+
+        # Look for expression of form
+        # c * cosh(s * T) + d * sinh(s * T) / (a * cosh(s * T) + b * sinh(s * T))
+
+        numer, denom = expr.as_numer_denom()
+
+        arg = None
+        a = Zero
+        b = Zero
+        c = Zero
+        d = Zero
+
+        for term in denom.expand().as_ordered_terms():
+
+            const, e = factor_const(term, s)
+            if e.is_Function and e.func == sym.cosh:
+                if arg is None:
+                    arg = e.args[0]
+                elif arg != e.args[0]:
+                    self.error('Mismatch cosh arg')
+                a += const
+            elif e.is_Function and e.func == sym.sinh:
+                if arg is None:
+                    arg = e.args[0]
+                elif arg != e.args[0]:
+                    self.error('Mismatch sinh arg')
+                b += const
+            else:
+                self.error('Term does not include cosh or sinh')
+
+        for term in numer.expand().as_ordered_terms():
+
+            const, e = factor_const(term, s)
+            if e.is_Function and e.func == sym.cosh:
+                if arg is None:
+                    arg = e.args[0]
+                elif arg != e.args[0]:
+                    self.error('Mismatch cosh arg')
+                c += const
+            elif e.is_Function and e.func == sym.sinh:
+                if arg is None:
+                    arg = e.args[0]
+                elif arg != e.args[0]:
+                    self.error('Mismatch sinh arg')
+                d += const
+            else:
+                self.error('Term does not include cosh or sinh')
+
+        T = arg / s
+        if T.has(s):
+            self.error('Frequency dependent delay')
+
+        Z0 = sym.sqrt((b * d - a * c) * d / (d**2 - c**2))
+        Zl = Z0 * c / d
+        Zs = (a - Z0 * Zl) / Z0
+        K = d / Z0**2
+        # The simplify can be slow
+        Gammas = ((Zs - Z0) / (Zs + Z0)).simplify()
+        Gammal = ((Zl - Z0) / (Zl + Z0)).simplify()
+
+        m = self.dummy_var(expr, 'm', level=0, real=True)
+        h1 = (1 - Gammas) / 2 * sym.DiracDelta(t)
+        h2 = (1 - Gammas) * (1 + Gammas) / (2 * Gammas) * sym.Sum((Gammas * Gammal)**m * sym.DiracDelta(t - 2 * m * T), (m, 1, sym.oo))
+        return K * (h1 + h2)
+
     def term1(self, expr, s, t, **kwargs):
 
         const, expr = factor_const(expr, s)
@@ -321,10 +434,37 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         except:
             pass
 
+        if expr.is_Pow and expr.args[1] == -1 and expr.args[0].is_Function:
+            arg = expr.args[0].args[0]
+
+            m = self.dummy_var(expr, 'm', level=0, real=True)
+            if expr.args[0].func == sym.cosh:
+                scale, shift = scale_shift(arg, s)
+                if shift == 0:
+                    return const * 2 * sym.Sum((-1)**m * sym.DiracDelta(t - scale * (2 * m + 1)), (m, 0, sym.oo)), Zero
+            elif expr.args[0].func == sym.sinh:
+                scale, shift = scale_shift(arg, s)
+                if shift == 0:
+                    return const * 2 * sym.Sum(sym.DiracDelta(t - scale * (2 * m + 1)), (m, 0, sym.oo)), Zero
+            elif expr.args[0].func == sym.tanh:
+                scale, shift = scale_shift(arg, s)
+                if shift == 0:
+                    return const * 2 * sym.Sum((-1)**m * sym.DiracDelta(t - scale * (2 * m + 1)), (m, 1, sym.oo)) + const * sym.DiracDelta(t), Zero
+
+        if expr.has(sym.cosh) and expr.has(sym.sinh):
+            try:
+                return const * self.tline_end(expr, s, t), Zero
+            except:
+                pass
+            try:
+                return const * self.tline_start(expr, s, t), Zero
+            except:
+                pass
+
         if expr.is_Pow and expr.args[0] == s:
             return Zero, const * self.power(expr, s, t)
 
-        raise ValueError('Cannot determine inverse Laplace transform')
+        self.error('Cannot determine inverse Laplace transform')
 
     def term(self, expr, s, t, **kwargs):
 
@@ -399,7 +539,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
                 cresult += uresult * sym.Heaviside(t - delay)
                 uresult = Zero
             else:
-                raise ValueError('Causality violated with time advance %s.' % delay)
+                self.error('Causality violated with time advance %s.' % delay)
 
         else:
             if kwargs.get('causal', False):
