@@ -6,6 +6,8 @@ from .state import state
 from .expr import Expr
 import sympy as sym
 import numpy as np
+from warnings import warn
+
 
 class NoiseExpression(Expr):
     """Frequency domain (one-sided) noise spectrum expression (amplitude
@@ -204,17 +206,18 @@ class NoiseExpression(Expr):
         return y
 
     def time(self):
-        print('Warning: no time representation for noise expression, '
-              'assumed zero: use rms()')
+        if False:
+            warn('No time representation for noise expression, '
+                 'assumed zero: use rms()')
         return 0
 
     def fourier(self):
-        print('Warning: no Fourier representation for noise expression, '
+        warn('No Fourier representation for noise expression, '
               'assumed zero: use asd() or psd()')
         return 0    
 
     def laplace(self):
-        print('Warning: no Laplace representation for noise expression, '
+        warn('No Laplace representation for noise expression, '
               'assumed zero')
         return 0
 
