@@ -129,9 +129,14 @@ class LcapyLatexPrinter(LatexPrinter):
     def _print(self, expr, exp=None):
 
         from .expr import Expr
+        from .sequence import Sequence
+        
         if isinstance(expr, Expr):        
             expr = expr.expr
 
+        if isinstance(expr, Sequence):
+            return self._print_Sequence(expr)
+            
         # Convert sym.I to j etc.
         try:
             if expr in latex_expr_map:
@@ -143,10 +148,33 @@ class LcapyLatexPrinter(LatexPrinter):
             return super(LcapyLatexPrinter, self)._print(expr)
         return super(LcapyLatexPrinter, self)._print(expr, exp=exp)        
 
+    def _print_Sequence(self, seq):
+
+        a = seq.zeroextend()
+
+        items = []
+        if seq.start_trunc:
+            items.append('\\ldots')
+            
+        for v1, n1 in zip(a, a.n):
+            try:
+                s = v1.latex()
+            except:
+                s = str(v1)
+            
+            if n1 == 0:
+                s = r'\underline{%s}' % v1
+            items.append(s)
+
+        if seq.end_trunc:
+            items.append('\\ldots')
+            
+        return r'\left\{%s\right\}' % ', '.join(items)
+    
     def _print_Piecewise(self, expr):
 
         if len(expr.args) > 1:
-            return super(LcapyLatexPrinter, self)._print(expr)                    
+            return super(LcapyLatexPrinter, self)._print_Piecewise(expr)                    
 
         e, c = expr.args[0]
         return  r"%s \;\; \text{for}\: %s" % (self._print(e), self._print(c))
@@ -173,6 +201,20 @@ class LcapyLatexPrinter(LatexPrinter):
             tex = r"u\left[%s\right]^{%s}" % (tex, exp)
         return tex
 
+    def _print_dtrect(self, expr, exp=None):
+
+        tex = r"\mathop{\mathrm{rect}}\left[%s\right]" % self._print(expr.args[0])
+        if exp:
+            tex = r"\mathop{\mathrm{rect}}\left[%s\right]^{%s}" % (tex, exp)
+        return tex
+
+    def _print_dtsign(self, expr, exp=None):
+
+        tex = r"\mathop{\mathrm{sign}}\left[%s\right]" % self._print(expr.args[0])
+        if exp:
+            tex = r"\mathop{\mathrm{sign}}\left[%s\right]^{%s}" % (tex, exp)
+        return tex        
+
     def _print_symbol_name(self, name):
 
         name = canonical_name(name)
@@ -274,7 +316,7 @@ class LcapyPrettyPrinter(PrettyPrinter):
         from sympy.printing.pretty.stringpict import prettyForm
         
         if len(expr.args) > 1:
-            return super(LcapyPrettyPrinter, self)._print(expr)
+            return super(LcapyPrettyPrinter, self)._print_Piecewise(expr)
 
         ec = expr.args[0]
 
@@ -318,7 +360,31 @@ class LcapyPrettyPrinter(PrettyPrinter):
             pform = prettyForm(*pform.left('u'))
             return pform
         else:
-            return self._print_Function(expr)        
+            return self._print_Function(expr)
+
+    def _print_dtrect(self, expr):
+
+        from sympy.printing.pretty.stringpict import prettyForm
+        
+        if self._use_unicode:
+            pform = self._print(expr.args[0])
+            pform = prettyForm(*pform.parens(left='[', right=']'))
+            pform = prettyForm(*pform.left('rect'))
+            return pform
+        else:
+            return self._print_Function(expr)
+
+    def _print_dtsign(self, expr):
+
+        from sympy.printing.pretty.stringpict import prettyForm
+        
+        if self._use_unicode:
+            pform = self._print(expr.args[0])
+            pform = prettyForm(*pform.parens(left='[', right=']'))
+            pform = prettyForm(*pform.left('sign'))
+            return pform
+        else:
+            return self._print_Function(expr)                
     
     
 def print_str(expr):
