@@ -1,14 +1,15 @@
 """
 This module provides support for rational functions.
 
-Copyright 2016--2020 Michael Hayes, UCECE
+Copyright 2016--2022 Michael Hayes, UCECE
 """
 
 from __future__ import division
 import sympy as sym
 from .sym import sympify, AppliedUndef
-from .cache import lru_cache
-from .utils import pair_conjugates
+from .cache import lru_cache, cached_property
+from .utils import pair_conjugates, factor_const
+from warnings import warn
 
 Zero = sym.S.Zero
 One = sym.S.One
@@ -112,27 +113,6 @@ class Pole(object):
             raise ValueError('Unknown damping %s' % self.damping)
 
 
-def as_numer_denom_poly(expr, var):
-
-    N = One
-    D = One
-    for f in expr.as_ordered_factors():
-        if f.is_Pow and f.args[1] == -1:
-            D *= f.args[0]
-        else:
-            N *= f
-
-    try:
-        Dpoly = sym.Poly(D, var)
-        Npoly = sym.Poly(N, var)
-    except:
-        N, D = expr.as_numer_denom()
-        Dpoly = sym.Poly(D, var)
-        Npoly = sym.Poly(N, var)
-
-    return Npoly, Dpoly
-
-
 def as_numer_denom(expr, var):
 
     if expr.has(1 / var):
@@ -234,45 +214,11 @@ def _pr2tf(poles, residues, var=None):
     return sym.Add(*[r / (var - p) for r, p in zip(residues, poles)], evaluate=False)
 
 
-def as_ratfun_delay(expr, var):
-    delay = Zero
+def as_B_A_delay_undef(expr, var):
 
-    if expr.is_rational_function(var):
-        N, D = expr.as_numer_denom()
-        return N, D, delay
-
-    F = sym.factor(expr).as_ordered_factors()
-
-    rf = One
-    for f in F:
-        b, e = f.as_base_exp()
-        if b == sym.E and e.is_polynomial(var):
-            p = sym.Poly(e, var)
-            c = p.all_coeffs()
-            if p.degree() == 1:
-                delay -= c[0]
-                if c[1] != 0:
-                    rf *= sym.exp(c[1])
-                continue
-
-        rf *= f
-
-    if not rf.is_rational_function(var):
-        raise ValueError('Expression not a product of rational function'
-                         ' and exponential')
-
-    N, D = rf.as_numer_denom()
-    return N, D, delay
-
-
-def as_ratfun_delay_undef(expr, var):
     delay = Zero
     undef = One
 
-    if expr.is_rational_function(var):
-        N, D = as_numer_denom(expr, var)
-        return N, D, delay, undef
-
     F = sym.factor(expr).as_ordered_factors()
 
     rf = One
@@ -303,74 +249,88 @@ def as_ratfun_delay_undef(expr, var):
 class Ratfun(object):
 
     def __init__(self, expr, var):
-        # Don't use cancel, it can cause a mess when have exp(s * T).
+        """This represents a generalized rational function of the form:
+        `H(var) = N(var) / D(var) = (B(var) / A(var)) * exp(-var * delay) * U(var)`
+
+        where B and D are polynomials in var and U is the product of
+        undefined functions of var.
+
+        Note, delay only represents a delay when var is s.
+
+        """
+
         self.expr = expr
         self.var = var
 
-    def as_ratfun_delay(self):
-        """Split expr as (N, D, delay)
-        where expr = (N / D) * exp(var * delay)
+        self.B, self.A, self.delay, self.undef = as_B_A_delay_undef(expr, var)
 
-        Note, delay only represents a delay when var is s."""
+        N = self.B * self.undef
+        if self.delay != 0:
+            N *= sym.exp(-self.delay * var)
 
-        return as_ratfun_delay(self.expr, self.var)
+        # These may change...  Perhaps, A should not have a constant factor?
+        self.N = N
+        self.D = self.A
 
-    def as_ratfun_delay_undef(self):
-        """Split expr as (N, D, delay, undef)
-        where expr = (N / D) * exp(var * delay) * undef
-        and where N is a polynomial in var,
-        D is a polynomial in var, and undef is the product
-        of undefined functions, e.g., V(s).
+    def as_B_A_delay_undef(self):
 
-        Note, delay only represents a delay when var is s."""
+        return self.B, self.A, self.delay, self.undef
 
-        return as_ratfun_delay_undef(self.expr, self.var)
+    def _roots(self, poly):
 
-    def as_const_undef_rest(self):
-        """Split expr as (const, undef, rest)
-        where expr = const * undef * rest"""
+        roots = sym.roots(poly)
+        num_roots = 0
+        for root, n in roots.items():
+            num_roots += n
+        if num_roots != poly.degree():
+            # When the degree is five or above, the roots
+            # cannot be found, see Abel-Ruffini theorem.
+            # If the coefficients of the polynomial are numerical,
+            # the SymPy nroots function can be used to find
+            # numerical approximations to the roots.
+            a = set()
+            a.add(self.var)
+            if poly.free_symbols == a:
+                warn('Only %d of %d roots found, using numerical approximation' % (num_roots, poly.degree()))
 
-        expr = self.expr
-        var = self.var
+                nroots = poly.nroots()
 
-        const = One
-        undef = One
-        rest = One
+                roots = {}
+                for root in nroots:
+                    if root in roots:
+                        roots[root] += 1
+                    else:
+                        roots[root] = 1
 
-        F = sym.factor(expr).as_ordered_factors()
+                return roots
+            warn('Only %d of %d roots found' % (num_roots, poly.degree()))
 
-        for f in F:
-            if isinstance(f, AppliedUndef):
-                undef *= f
-                continue
-            if not f.has(var):
-                const *= f
-                continue
-            rest *= f
-
-        return const, undef, rest
+        return roots
 
     @lru_cache()
     def roots(self):
         """Return roots of expression as a dictionary
         Note this may not find them all."""
 
-        return sym.roots(sym.Poly(self.expr, self.var))
+        return self._roots(sym.Poly(self.expr, self.var))
 
     @lru_cache()
     def zeros(self):
         """Return zeroes of expression as a dictionary
         Note this may not find them all."""
 
-        return Ratfun(self.numerator, self.var).roots()
+        return self._roots(self.Bpoly)
 
     @lru_cache()
     def poles(self, damping=None):
-        """Return poles of expression as a dictionary of Pole objects.
+        """Return poles of expression as a list of Pole objects.
         Note this may not find all the poles."""
 
+        roots = self._roots(self.Apoly)
+
         poles = []
-        for p, n in Ratfun(self.denominator, self.var).roots().items():
+
+        for p, n in roots.items():
 
             pole = Pole(p, n=n, damping=damping)
             for q in poles:
@@ -435,33 +395,13 @@ class Ratfun(object):
         m2 = method2(numer, denom, var, pole)
         return m2.cancel()
 
-    @property
-    def numerator_denominator(self):
-        """Return numerator and denominator of rational function"""
+    @cached_property
+    def Bpoly(self):
+        return sym.Poly(self.B, self.var)
 
-        return as_numer_denom(self.expr, self.var)
-
-    @property
-    def N(self):
-        return self.numerator
-
-    @property
-    def D(self):
-        return self.denominator
-
-    @property
-    def numerator(self):
-        """Return numerator of rational function"""
-
-        numer, denom = self.numerator_denominator
-        return numer
-
-    @property
-    def denominator(self):
-        """Return denominator of rational function"""
-
-        numer, denom = self.numerator_denominator
-        return denom
+    @cached_property
+    def Apoly(self):
+        return sym.Poly(self.A, self.var)
 
     def canonical(self, factor_const=True):
         """Convert rational function to canonical form; this is like general
@@ -478,24 +418,19 @@ class Ratfun(object):
 
         """
 
-        try:
-            N, D, delay, undef = self.as_ratfun_delay_undef()
-        except ValueError:
-            # TODO: copy?
-            return self.expr
-
         var = self.var
-        Dpoly = sym.Poly(D, var)
-        Npoly = sym.Poly(N, var)
+        Apoly = self.Apoly
+        Bpoly = self.Bpoly
+        delay = self.delay
 
         if factor_const:
-            K = sym.cancel(Npoly.LC() / Dpoly.LC())
+            K = sym.cancel(Bpoly.LC() / Apoly.LC())
             if delay != 0:
                 K *= sym.exp(self.var * delay)
 
             # Divide by leading coefficient
-            N = Npoly.monic().as_expr()
-            D = Dpoly.monic().as_expr()
+            N = Bpoly.monic().as_expr()
+            D = Apoly.monic().as_expr()
 
             if D == 1:
                 expr = N
@@ -504,11 +439,11 @@ class Ratfun(object):
 
             if K != 1:
                 expr = sym.Mul(K, expr, evaluate=False)
-            expr *= undef
+            expr *= self.undef
         else:
-            C = Dpoly.LC()
-            D = Dpoly.monic().as_expr()
-            N = (Npoly.as_expr() / C).simplify()
+            C = Apoly.LC()
+            D = Apoly.monic().as_expr()
+            N = (Bpoly.as_expr() / C).simplify()
             if D == 1:
                 expr = N
             else:
@@ -518,7 +453,7 @@ class Ratfun(object):
                     expr = sym.Mul(N, 1 / D, evaluate=False)
             if delay != 0:
                 expr *= sym.exp(self.var * delay)
-            expr *= undef
+            expr *= self.undef
 
         return expr
 
@@ -527,9 +462,9 @@ class Ratfun(object):
 
         See also canonical, partfrac, standard, timeconst, and ZPK"""
 
-        N, D, delay, undef = self.as_ratfun_delay_undef()
+        B, A, delay, undef = self.as_B_A_delay_undef()
 
-        expr = sym.cancel(N / D, self.var)
+        expr = sym.cancel(B / A, self.var)
         if delay != 0:
             expr *= sym.exp(self.var * delay)
 
@@ -539,14 +474,14 @@ class Ratfun(object):
         """Expand in terms for different powers with each term
         expressed in canonical form."""
 
-        N, D, delay, undef = self.as_ratfun_delay_undef()
+        B, A, delay, undef = self.as_B_A_delay_undef()
 
-        Npoly = sym.Poly(N, self.var)
+        Bpoly = self.Bpoly
 
         expr = Zero
 
-        for m, c in enumerate(reversed(Npoly.all_coeffs())):
-            term = sym.Mul(c.simplify() * self.var ** m, 1 / D)
+        for m, c in enumerate(reversed(Bpoly.all_coeffs())):
+            term = sym.Mul(c.simplify() * self.var ** m, 1 / A)
             expr += term
 
         if delay != 0:
@@ -554,7 +489,7 @@ class Ratfun(object):
 
         return expr * undef
 
-    def partfrac(self, combine_conjugates=False, damping=None, split=True):
+    def partfrac(self, combine_conjugates=False, damping=None, method=None):
         """Convert rational function into partial fraction form.
 
         If combine_conjugates is True then the pair of partial
@@ -564,22 +499,11 @@ class Ratfun(object):
 
         """
 
-        try:
-            Q, R, D, delay, undef = self.as_QRD(combine_conjugates, damping)
-        except ValueError:
-            if not split:
-                raise
-
-            # Try splitting into terms
-            result = 0
-            for term in self.expr.as_ordered_terms():
-                result += Ratfun(term, self.var).partfrac(combine_conjugates,
-                                                          split=False)
-            return result
+        Q, R, F, delay, undef = self.as_QRF(combine_conjugates, damping, method)
 
         result = Q
-        for R, D in zip(R, D):
-            result += R / D
+        for R, F in zip(R, F):
+            result += R / F
 
         if delay != 0:
             result *= sym.exp(-self.var * delay)
@@ -587,7 +511,7 @@ class Ratfun(object):
         result *= undef
         return result
 
-    def standard(self, split=True):
+    def standard(self):
         """Convert rational function into mixed fraction form.
 
         This is the sum of strictly proper rational function and a
@@ -595,19 +519,9 @@ class Ratfun(object):
 
         See also canonical, general, partfrac, timeconst, and ZPK"""
 
-        try:
-            Q, M, D, delay, undef = self.as_QMD()
-        except ValueError:
-            if not split:
-                raise
+        Q, M, A, delay, undef = self.as_QMA()
 
-            # Try splitting into terms
-            result = 0
-            for term in self.expr.as_ordered_terms():
-                result += Ratfun(term, self.var).standard(False)
-            return result
-
-        expr = Q + sym.cancel(M / D, self.var)
+        expr = Q + sym.cancel(M / A, self.var)
 
         if delay != 0:
             expr *= sym.exp(-self.var * delay)
@@ -620,17 +534,16 @@ class Ratfun(object):
 
         See also canonical, general, partfrac, standard, and ZPK"""
 
-        N, D, delay, undef = self.as_ratfun_delay_undef()
+        B, A, delay, undef = self.as_B_A_delay_undef()
 
         var = self.var
-        Npoly = sym.Poly(N, var)
-        Dpoly = sym.Poly(D, var)
+        Apoly = self.Apoly
 
-        K = Dpoly.EC()
+        K = Apoly.EC()
 
-        D = D / K
-        N = N / K
-        return sym.Mul(N, sym.Pow(D, -1), evaluate=False) * sym.exp(self.var * delay) * undef
+        A = A / K
+        B = B / K
+        return sym.Mul(B, sym.Pow(A, -1), evaluate=False) * sym.exp(self.var * delay) * undef
 
     def ZPK(self, combine_conjugates=False):
         """Convert to zero-pole-gain (ZPK) form.
@@ -669,20 +582,17 @@ class Ratfun(object):
         """Return residues of partial fraction expansion.
 
         This is not much use without the corresponding poles.
-        It is better to use as_QRD."""
+        It is better to use as_QRF."""
 
-        Q, R, D, delay, undef = self.as_QRD(combine_conjugates, damping)
+        Q, R, F, delay, undef = self.as_QRF(combine_conjugates, damping)
         return R
 
     def coeffs(self):
 
-        N, D, delay, undef = self.as_ratfun_delay_undef()
+        Bpoly = self.Bpoly
+        Apoly = self.Apoly
 
-        var = self.var
-        Npoly = sym.Poly(N, var)
-        Dpoly = sym.Poly(D, var)
-
-        return Npoly.all_coeffs(), Dpoly.all_coeffs()
+        return Bpoly.all_coeffs(), Apoly.all_coeffs()
 
     @property
     def degree(self):
@@ -691,24 +601,21 @@ class Ratfun(object):
         This the maximum of the numerator and denominator degrees.
         Note zero has a degree of -inf."""
 
-        Npoly, Dpoly = as_numer_denom_poly(self.expr, self.var)
-        return max(Npoly.degree(), Dpoly.degree())
+        return max(self.Bpoly.degree(), self.Apoly.degree())
 
     @property
     def Ndegree(self):
         """Return the degree (order) of the numerator of a rational function.
         Note zero has a degree of -inf."""
 
-        Npoly, Dpoly = as_numer_denom_poly(self.expr, self.var)
-        return Npoly.degree()
+        return self.Bpoly.degree()
 
     @property
     def Ddegree(self):
         """Return the degree (order) of the denominator of a rational function.
         Note zero has a degree of -inf."""
 
-        Npoly, Dpoly = as_numer_denom_poly(self.expr, self.var)
-        return Dpoly.degree()
+        return self.Apoly.degree()
 
     @property
     def is_strictly_proper(self):
@@ -723,41 +630,38 @@ class Ratfun(object):
         expression = K * (prod_n (var - z_n) / (prod_n (var - p_n)) * undef
         """
 
-        N, D, delay, undef = self.as_ratfun_delay_undef()
+        Bpoly = self.Bpoly
+        Apoly = self.Apoly
 
-        var = self.var
-        Npoly = sym.Poly(N, var)
-        Dpoly = sym.Poly(D, var)
+        K = sym.cancel(Bpoly.LC() / Apoly.LC())
+        if self.delay != 0:
+            K *= sym.exp(self.var * self.delay)
 
-        K = sym.cancel(Npoly.LC() / Dpoly.LC())
-        if delay != 0:
-            K *= sym.exp(self.var * delay)
+        zeros = sym.roots(Bpoly)
+        poles = sym.roots(Apoly)
 
-        zeros = sym.roots(Npoly)
-        poles = sym.roots(Dpoly)
+        return zeros, poles, K, self.undef
 
-        return zeros, poles, K, undef
+    def as_QMA(self):
+        """Decompose expression into Q, M, A, delay, undef where
 
-    def as_QMD(self):
-        """Decompose expression into Q, M, D, delay, undef where
+        `expression = (Q + M / A) * exp(-delay * var) * undef`"""
 
-        expression = (Q + M / D) * exp(-delay * var) * undef"""
+        B, A, delay, undef = self.as_B_A_delay_undef()
 
-        N, D, delay, undef = self.as_ratfun_delay_undef()
+        # Perform polynomial long division so expr = Q + M / A
+        Q, M = sym.div(B, A, self.var)
 
-        # Perform polynomial long division so expr = Q + M / D
-        Q, M = sym.div(N, D, self.var)
+        return Q, M, A, delay, undef
 
-        return Q, M, D, delay, undef
+    def as_QRF_old(self, combine_conjugates=False, damping=None):
+        """Decompose expression into Q, R, F, delay, undef where
 
-    def as_QRD_old(self, combine_conjugates=False, damping=None):
-        """Decompose expression into Q, R, D, delay, undef where
+        expression = (Q + sum_n r_n / f_n) * exp(-delay * var) * undef"""
 
-        expression = (Q + sum_n R_n / D_n) * exp(-delay * var) * undef"""
+        Q, M, F, delay, undef = self.as_QMA()
 
-        Q, M, D, delay, undef = self.as_QMD()
-
-        expr = M / D
+        expr = M / F
         var = self.var
 
         sexpr = Ratfun(expr, var)
@@ -848,17 +752,137 @@ class Ratfun(object):
                 Onew.append(o)
         return Rnew, Pnew, Onew
 
-    def as_QRPO(self, damping=None):
-        """Decompose expression into Q, R, P, O, delay, undef where
-
-        expression = (Q + sum_n R_n / (var - P_n)**O_n) * exp(-delay * var) * undef
-        """
+    def _find_residues_ec(self, poles, B):
 
         from .matrix import matrix_inverse
 
-        Q, M, D, delay, undef = self.as_QMD()
+        # Find residues by solving system of equations (equating coefficients
+        # method).  For example,  consider an expression with repeated poles:
+        # E(s) = B(S) / (2 * (s - p1)**2 * (s - p2))
+        #
+        # This can be expressed as partial fractions:
+        # E(s) = R_1 / (s - p1)**2 + R_2 / (s - p1) + R_3 / (s - p2)
+        # where R_n are the residues we would like to find.
+        #
+        # The factored denominator expression is
+        # Af(s) = ((s - p1)**2 * (s - p2)) = A(s) / C where C is a constant
+        # and so the numerator expression is
+        # B(s) = E(s) * A(s) = E(s) * Af(s) * C
+        #
+        # Expanding the partial fractions to have a common denominator gives:
+        # B(s) = R_1 * (s - p2) + R_2 * (s - p1) * (s - p2) + R * (s - p1)**2
+        #
+        # Equating B(s) with E(s) * A(s) and equating powers of s gives
+        # system of equation to find the residues.
 
-        expr = M / D
+        var = self.var
+
+        U = []
+        F = []
+        O = []
+        P = []
+        i = 1
+        A_factored = One
+        for pole in poles:
+            p = pole.expr
+            f = var - p
+
+            for m in range(pole.n):
+                u = sym.Symbol('r_%d' % i)
+                o = pole.n - m
+                U.append(u)
+                F.append(f)
+                P.append(p)
+                O.append(o)
+                i += 1
+                A_factored *= f
+
+        # This is slow due to use of cancel
+        # rhs = Zero
+        # for f, o, u in zip(F, O, U):
+             # rhs += u * (A_factored / f**o).cancel()
+
+        rhs = Zero
+        for i in range(len(U)):
+            x = U[i]
+            for j in range(len(U)):
+                if i == j:
+                    continue
+                if F[i] is not F[j] or O[j] > O[i]:
+                    x *= F[j]
+            rhs += x
+
+        lhs = B
+
+        rhspoly = sym.poly(rhs, var)
+        lhspoly = sym.poly(lhs, var)
+
+        rc = rhspoly.all_coeffs()
+        lc = lhspoly.all_coeffs()
+        if len(lc) < len(rc):
+            lc = [0] * (len(rc) - len(lc)) + lc
+
+        A, _ = sym.linear_eq_to_matrix(rc, U)
+
+        # Solve system of equations to find residues.
+        R = list(matrix_inverse(A) * sym.Matrix(lc))
+
+        return R, P, O
+
+    def _find_residues_sub(self, poles, B):
+
+        var = self.var
+
+        F = []
+        O = []
+        R = []
+        P = []
+        M = []
+
+        for pole in poles:
+            p = pole.expr
+            f = var - p
+
+            for m in range(pole.n):
+                o = pole.n - m
+                F.append(f)
+                P.append(p)
+                O.append(o)
+                M.append(pole.n)
+
+        for i in range(len(P)):
+
+            if M[i] == O[i]:
+                denom = One
+                for j in range(len(P)):
+
+                    if i == j:
+                        continue
+                    if F[i] is not F[j] or O[j] > O[i]:
+                        denom *= F[j]
+                expr = B / denom
+                r = expr.subs(var, P[i])
+                R.append(r)
+            else:
+                expr = expr.diff(var)
+                r = expr.subs(var, P[i]) / sym.factorial(M[i] - O[i])
+                R.append(r)
+
+        return R, P, O
+
+    def as_QRPO(self, damping=None, method=None):
+        """Decompose expression into Q, R, P, O, delay, undef where
+
+        `expression = (Q + sum_n r_n / (var - p_n)**o_n) * exp(-delay * var) * undef`
+
+        `method` can be 'sub' (substitution method, the default) or
+        'ec' (equating cofficients method).a
+
+        """
+
+        Q, M, A, delay, undef = self.as_QMA()
+
+        expr = M / A
         var = self.var
 
         sexpr = Ratfun(expr, var)
@@ -879,53 +903,25 @@ class Ratfun(object):
             r = (expr * d).cancel()
             return Q, [r], [p], [1], delay, undef
 
-        # Find residues by solving system of equations.
+        B = sexpr.B
+        B /= sexpr.Apoly().LC()
 
-        syms = []
-        D = []
-        O = []
-        P = []
-        i = 1
-        denom_factored = One
-        for pole in poles:
-            for m in range(pole.n):
-                A = sym.Symbol('A_%d' % i)
-                p = pole.expr
-                d = (var - p) ** (m + 1)
-                syms.append(A)
-                D.append(d)
-                P.append(p)
-                O.append(m + 1)
-                i += 1
-            denom_factored *= (var - pole.expr) ** pole.n
-
-        rhs = Zero
-        for denom, residue in zip(D, syms):
-            # Could be more cunning to avoid using cancel
-            rhs += residue * (denom_factored / denom).cancel()
-
-        rhspoly = sym.poly(rhs, var)
-        lhspoly = sym.poly((expr * denom_factored).cancel(), var)
-
-        rc = rhspoly.all_coeffs()
-        lc = lhspoly.all_coeffs()
-        if len(lc) < len(rc):
-            lc = [0] * (len(rc) - len(lc)) + lc
-
-        A, _ = sym.linear_eq_to_matrix(rc, syms)
-
-        # Solve system of equations to find residues A_n.
-        R = list(matrix_inverse(A) * sym.Matrix(lc))
+        if method in ('sub', None):
+            R, P, O = self._find_residues_sub(poles, B)
+        elif method == 'ec':
+            R, P, O = self._find_residues_ec(poles, B)
+        else:
+            raise ValueError('Unknown method ' + method)
 
         # Remove elements where the residue is zero.
         R, P, O = self._prune_zero_residues(R, P, O)
 
         return Q, R, P, O, delay, undef
 
-    def _combine_conjugates(self, R, D, P, O):
+    def _combine_conjugates(self, R, F, P, O):
 
         Rnew = []
-        Dnew = []
+        Fnew = []
 
         for m in range(len(R)):
 
@@ -949,23 +945,23 @@ class Ratfun(object):
 
             if has_conjugate:
                 Rnew.append(((self.var - pc) * r + (self.var - p) * rc).expand())
-                Dnew.append(((self.var - p) * (self.var - pc)).expand())
+                Fnew.append(((self.var - p) * (self.var - pc)).expand())
             else:
                 Rnew.append(r)
-                Dnew.append(D[m])
-        return Rnew, Dnew
+                Fnew.append(F[m])
+        return Rnew, Fnew
 
 
-    def as_QRD(self, combine_conjugates=False, damping=None):
-        """Decompose expression into Q, R, D, delay, undef where
+    def as_QRF(self, combine_conjugates=False, damping=None, method=None):
+        """Decompose expression into Q, R, F, delay, undef where
 
-        expression = (Q + sum_n R_n / D_n) * exp(-delay * var) * undef"""
+        expression = (Q + sum_n r_n / f_n) * exp(-delay * var) * undef"""
 
-        Q, R, P, O, delay, undef = self.as_QRPO(damping)
+        Q, R, P, O, delay, undef = self.as_QRPO(damping, method)
 
-        D = [(self.var - p)**o for p, o in zip(P, O)]
+        F = [(self.var - p)**o for p, o in zip(P, O)]
 
         if combine_conjugates:
-            R, D = self._combine_conjugates(R, D, P, O)
+            R, F = self._combine_conjugates(R, F, P, O)
 
-        return Q, R, D, delay, undef
+        return Q, R, F, delay, undef
