@@ -1,42 +1,38 @@
 """This module provides classes for schematic layout.
 
-Copyright 2014--2020 Michael Hayes, UCECE
+Copyright 2014--2021 Michael Hayes, UCECE
 
 """
+from .config import colours
+from .cnodes import Cnodes
 
+# The component placement algorithm is similar to a critical path
+# method (CPM).  For most netlists it does a good job.  However, there
+# are some loop constructs where it fails.
 
-from __future__ import print_function
+# An alternative approach is to form a system of linear equations
+# where the unknowns are the component positions and the component
+# stretch.  Most of the component stretches will be free variables and
+# are set to zero stretch.  However, this algorithm will also fail for
+# the same loop constructs as the graph algorithm,
 
-def unique(alist):
+# Here's a problem example:
+#
+# R1 1 2; right=2
+# R2 2 3; right=1
+# W 2 5; down=1
+# W 4 5; right=0.5
+# W 5 6; right=0.5
+# C 4 7; down
+# R 6 9; down
+# W 7 8; right=0.5
+# W 8 9; right=0.5
+# W 8 11; down=1
+# W 10 11; right
+# W 11 12; right=2
 
-    # Order preserving...  list(set(alist)) gives different results
-    # for different runs.
-    used = set()
-    return [x for x in alist if x not in used and (used.add(x) or True)]
-
-
-class Cnodes(dict):
-    """Common nodes"""
-
-    def __init__(self, nodes):
-
-        super (Cnodes, self).__init__()
-        for node in nodes:
-            # Use tuple so hashable.
-            self[node] = (node, )
-
-    def link(self, n1, n2):
-        """Make nodes n1 and n2 share common node"""
-
-        set1 = self[n1]
-        set2 = self[n2]
-
-        newset = tuple(unique(set1 + set2))
-
-        for n in self[n1]:
-            self[n] = newset
-        for n in self[n2]:
-            self[n] = newset
+# For the hotizontal graph, the critical path is through nodes 1 and
+# 12 but this not found due to the loop for nodes 2, 4, 6, and 8.
 
 
 class Gedge(object):
@@ -196,7 +192,7 @@ class GraphPath(list):
 
         return self[0].from_gnode
 
-                
+
 class Graph(dict):
     """Drawing graph"""
 
@@ -271,7 +267,7 @@ class Graph(dict):
 
     @property
     def all_nodes(self):
-        return unique(self.cnodes.values())
+        return self.cnodes.all_nodes()
 
     def add_start_nodes(self):
         """Nodes without forward edges are connected to the end node.
@@ -345,13 +341,13 @@ class Graph(dict):
             # unlucky gnode. 
             return False
 
-        if from_gnode.name == 'start' and len(to_path) == 1:
+        if from_gnode.name == 'start':
             # Have dangling node, so no stretch needed.
             gnode.pos = to_gnode.pos - to_path.dist
             unknown.remove(gnode.name)
             return True
 
-        if to_gnode.name == 'end' and len(from_path) == 1:
+        if to_gnode.name == 'end':
             # Have dangling node, so no stretch needed.
             gnode.pos = from_gnode.pos + from_path.dist
             unknown.remove(gnode.name)
@@ -359,32 +355,45 @@ class Graph(dict):
 
         path = self.longest_path(from_gnode, to_gnode)
 
-        if len(path) == 1:
-            return False
+        #if len(path) == 1:
+        #    return False
         
         # The gnode may not be on this path but this does not matter.
-        # It will be processed asain later.
+        # It will be processed again later.
 
         stretches = path.stretches
         separation = to_gnode.pos - from_gnode.pos
         extent = path.dist        
 
         if extent - separation > 1e-6:
-            raise ValueError('Inconsistent %s schematic graph, component(s) will not fit:  separation %s between %s and %s, need %s.\n%s' % (self.name, separation, from_gnode, to_gnode, extent, self))
-
+            print('Inconsistent %s schematic graph, component(s) will not fit:  separation %s between %s and %s, need %s.\n%s' % (self.name, separation, from_gnode, to_gnode, extent, self))
         
         # This how much each component needs to stretch.
-        stretch = (separation - extent) / stretches
+        if stretches == 0:
+            stretch = 0
+        else:            
+            stretch = (separation - extent) / stretches
+            if stretch < 0:
+                stretch = 0
 
         pos = from_gnode.pos
-        for edge in path:
+        for edge in reversed(from_path):
             pos += edge.size
             if edge.stretch:
                 pos += stretch
 
-            if edge != path[-1]:
+            if edge.from_gnode.pos == None:
+                edge.from_gnode.pos = pos
+                unknown.remove(edge.from_gnode.name)
+
+        for edge in to_path:
+            pos += edge.size
+            if edge.stretch:
+                pos += stretch
+
+            if edge.to_gnode.pos == None:
                 edge.to_gnode.pos = pos
-                unknown.remove(edge.to_gnode.name)
+                unknown.remove(edge.to_gnode.name)                
 
         return True
 
@@ -443,6 +452,11 @@ class Graph(dict):
                 for key, edges in fromto.items():
 
                     best_edge = edges[0]
+
+                    if len(edges) == 1:
+                        newedges.append(best_edge)                        
+                        continue
+                    
                     fixed = False
                     size = 0
                     for edge in edges:
@@ -472,19 +486,66 @@ class Graph(dict):
                     
 
             gnode.fedges = check(gnode.fedges, grizzle=True)
-            gnode.redges = check(gnode.redges)            
+            gnode.redges = check(gnode.redges)
 
-    def analyse(self, stage=None):
+    def suggest_edges(self):
+
+        for node in self.values():
+            fedges = node.fedges
+            redges = node.redges
+            if len(redges) != 0:
+                continue
+            if len(fedges) != 2:
+                continue
+            fedge1 = fedges[0]
+            fedge2 = fedges[1]            
+            
+            node1 = fedge1.to_gnode
+            node2 = fedge2.to_gnode
+            if len(node1.fedges) != 2:
+                continue
+            if len(node2.fedges) != 2:
+                continue
+            if node1.fedges[0].to_gnode == node2.fedges[0].to_gnode:
+                node3 = node1.fedges[0].to_gnode
+                fedge3 = node1.fedges[0]
+                fedge4 = node2.fedges[0]                
+            elif node1.fedges[0].to_gnode == node2.fedges[1].to_gnode:
+                node3 = node1.fedges[0].to_gnode
+                fedge3 = node1.fedges[0]
+                fedge4 = node2.fedges[1]                                
+            elif node1.fedges[1].to_gnode == node2.fedges[0].to_gnode:
+                node3 = node1.fedges[1].to_gnode
+                fedge3 = node1.fedges[1]
+                fedge4 = node2.fedges[0]                                
+            elif node1.fedges[1].to_gnode == node2.fedges[1].to_gnode:
+                node3 = node1.fedges[1].to_gnode
+                fedge3 = node1.fedges[1]
+                fedge4 = node2.fedges[1]                                
+            else:
+                continue
+            if len(node3.fedges) != 0:
+                continue
+            if fedge1.size == fedge3.size and fedge2.size == fedge4.size:
+                pass
+            elif fedge1.size == fedge2.size and fedge3.size == fedge4.size:
+                pass
+            else:
+                continue
+            
+            print('Suggest a constraint between nodes %s and %s for %s graph' % (node1, node2, self.name))
+
+    def solve(self, stage=None):
         """
-        Analyse graph assigning gnode positions.
+        Solve graph assigning gnode positions.
 
-        stage 0 --- add start/end gnodes
+        stage 0 --- do nothing
         stage 1 --- prune redundant edges
-        stage 2 --- assign gnode positions on longest path
+        stage 2 --- add start/end nodes and assign gnode positions on longest path
         stage 3 --- assign gnode positions with fixed positions to known gnodes
         stage 4 --- assign all gnode positions
         """
-        
+
         unknown = list(self.keys())
 
         if unknown == []:
@@ -494,19 +555,21 @@ class Graph(dict):
                 pos[n] = 0
             return pos, 0
         
-        self.add_start_nodes()
-        unknown.append('start')
-        unknown.append('end')        
-
         if stage == 0:
             return
 
         # Prune redundant edges from graph.        
         self.prune()
 
+        self.suggest_edges()
+        
         if stage == 1:
             return
-            
+
+        self.add_start_nodes()
+        unknown.append('start')
+        unknown.append('end')        
+        
         # Find longest path through the graph.  This provides the
         # dimension for the graph.
         path = self.longest_path(self['start'], self['end'])
@@ -548,7 +611,7 @@ class Graph(dict):
 
         return pos, distance_max
 
-    def longest_path(self, from_gnode, to_gnode, forward=True):
+    def longest_path(self, from_gnode, to_gnode):
         """Find longest path through DAG from from_gnode to to_gnode
         or to a gnode with a known position."""
 
@@ -574,9 +637,10 @@ class Graph(dict):
             if gnode.pos is not None and gnode != from_gnode:
                 return -1
 
+            # Mark node as visited but distance as unknown
             gnode.dist = -1
 
-            edges = gnode.fedges if forward else gnode.redges
+            edges = gnode.fedges
             for edge in edges:
                 dist = traverse(edge.to_gnode)
                 if dist >= 0 and gnode.dist < dist + edge.size:
@@ -669,7 +733,7 @@ class Graph(dict):
                                   edge.name, gnode.fmt_name,
                                   edge.to_gnode.fmt_name))
 
-    def dot(self, filename=None, stage=None):
+    def dot(self, filename=None, stage=None, tweak_node_label=True):
         """Generate directed graph using graphviz notation"""
 
         from .system import tmpfilename, run_dot
@@ -679,6 +743,19 @@ class Graph(dict):
         def fmt_dec(value):
             return ('%.2f' % value).rstrip('0').rstrip('.')
 
+        def fmt_node_label(node):
+
+            if isinstance(node.name, tuple):
+                label = ', '.join(node.name)
+            else:
+                label = node.name
+
+            # Hack since xlabel not supported bu dot2tex.
+            if tweak_node_label and node.pos is not None:
+                label += ' @' + fmt_dec(node.pos)
+
+            return label
+
         if filename is None:
             filename = tmpfilename('.png')
             self.dot(filename=filename, stage=stage)
@@ -692,30 +769,32 @@ class Graph(dict):
             run_dot(dotfilename, filename)
             return
 
-        self.analyse(stage=stage)
+        self.solve(stage=stage)
 
         dotfile = open(filename, 'w')
         dotfile.write('digraph {\n\tgraph [rankdir=LR];\n')
 
         for gnode in self.values():
             if hasattr(gnode, 'fixed'):
-                colour = 'yellow'
+                colour = colours['fixednode']
             else:
-                colour = 'red' if gnode.pos is not None else 'blue'
-            if gnode.name in ('start', 'end'):
-                colour = 'green'
+                colour = colours['assignednode'] if gnode.pos is not None else colours['unassignednode']
+            if gnode.name  == 'start':
+                colour = colours['startnode']
+            elif gnode.name  == 'end':
+                colour = colours['endnode']                
 
             pos = gnode.pos
             if pos is None or pos < 1e-6:
                 pos = 0
 
-            dotfile.write('\t"%s"\t [style=filled, color=%s, xlabel="@%s"];\n' % (gnode.fmt_name, colour, fmt_dec(pos)))
+            dotfile.write('\t"%s"\t [style=filled, color=%s];\n' % (fmt_node_label(gnode), colour))
 
         for gnode in self.values():
             for edge in gnode.fedges:
-                colour = 'black' if edge.stretch else 'red'
+                colour = colours['stretchedge'] if edge.stretch else colours['fixededge']
                 dotfile.write('\t"%s" ->\t"%s" [ color=%s, label="%s%s" ];\n' % (
-                    gnode.fmt_name, edge.to_gnode.fmt_name, colour, 
+                    fmt_node_label(gnode), fmt_node_label(edge.to_gnode), colour, 
                     fmt_dec(edge.size), '*' if edge.stretch else ''))
 
         dotfile.write('}\n')
