@@ -1,12 +1,13 @@
 """
 This module implements an experimental time-stepping simulation.
 
-Copyright 2020 Michael Hayes, UCECE
+Copyright 2021 Michael Hayes, UCECE
 """
 
 from numpy import zeros, array, float, linalg, dot
 from .sym import tsym, symbol_map
 from .symbols import oo
+from warnings import warn
 
 __all__ = ('Simulator', )
 
@@ -188,11 +189,11 @@ class SimulationResults(object):
             name = '%d' % name
 
         if name in cct.nodes:
-            return SimulationResultsNode(self.node_voltages_get(name))
+            return SimulationResultsNode(self._node_voltage_get(name))
         
         if name in cct._elements:
-            return SimulationResultsCpt(self.cpt_voltages_get(name),
-                                        self.cpt_currents_get(name))
+            return SimulationResultsCpt(self._cpt_voltage_get(name),
+                                        self._cpt_current_get(name))
 
         raise AttributeError('Unknown element or node name %s' % name)
 
@@ -208,63 +209,44 @@ class SimulationResults(object):
 
         return self.__getitem__(attr)
 
-    def node_voltages_get(self, n):
+    def _node_voltage_get(self, n):
 
-        index = self.r_model._node_index(n)
+        index = self.r_model.mna._node_index(n)
         # NB, node_voltages is zero for index = -1
         return self.node_voltages[index]
         
-    def cpt_voltages_get(self, cptname):
+    def _cpt_voltage_get(self, cptname):
 
         cpt = self.cct.elements[cptname]        
 
-        v1 = self.node_voltages_get(cpt.nodenames[0])
-        v2 = self.node_voltages_get(cpt.nodenames[1])        
+        v1 = self._node_voltage_get(cpt.nodenames[0])
+        v2 = self._node_voltage_get(cpt.nodenames[1])        
         return v1 - v2
 
-    def cpt_voltage_get(self, cptname, n):
-
-        cpt = self.cct.elements[cptname]
-        
-        v1 = self.node_voltages_get(cpt.nodenames[0])[n]
-        v2 = self.node_voltages_get(cpt.nodenames[1])[n]     
-        return v1 - v2
-
-    def cpt_currents_get(self, cptname):
+    def _cpt_current_get(self, cptname):
 
         try:
-            index = self.r_model._branch_index(cptname)
+            index = self.r_model.mna._branch_index(cptname)
             return self.branch_currents[index]
         except:
             cpt = self.cct._elements[cptname]
             if cpt.is_capacitor or cpt.is_inductor:
                 # For a capacitor we can find the current through the
                 # companion resistor or voltage source.
-                return self.cpt_currents_get('V%seq' % cptname)
+                return self._cpt_current_get('V%seq' % cptname)
 
-            Vd = self.cpt_voltages_get(cptname)            
+            Vd = self._cpt_voltage_get(cptname)            
             if cpt.is_resistor or cpt.is_conductor:
-                return Vd / cpt.R.expr
+                return Vd / float(cpt.R.expr)
                 
             # Need to determine resistance of the cpt
             raise ValueError('FIXME')            
 
-    def cpt_current_get(self, cptname, n):
-
-        return self.cpt_currents_get(cptname)[n]
-        
     @property
     def V(self, node):
         """Node voltage with respect to ground."""
 
-        return self.get_Vd(node, '0')
-
-
-    def get_Vd(self, Np, Nm=None):
-        """Voltage drop between nodes"""
-
-        self._solve()
-        return (self._Vdict[Np] - self._Vdict[Nm]).canonical()    
+        return self._node_voltage_get(node)
 
     
 class Simulator(object):
@@ -364,9 +346,6 @@ class Simulator(object):
 
         r_model = self.r_model
 
-        # Construct MNA matrices.
-        r_model._analyse()
-
         Asubsdict = {}
         Zsubsdict = {}        
         self.reactive_cpts = []        
@@ -375,13 +354,13 @@ class Simulator(object):
                 continue
 
             if not elt.has_ic:
-                print('Warning, initial conditions for %s ignored' % elt.name)
+                warn('Initial conditions for %s ignored' % elt.name)
             
-            v1_index = r_model._node_index(elt.nodenames[0])
-            v2_index = r_model._node_index(elt.nodenames[1])
-            i_index = r_model._branch_index('V%seq' % elt.name)
+            v1_index = r_model.mna._node_index(elt.nodenames[0])
+            v2_index = r_model.mna._node_index(elt.nodenames[1])
+            i_index = r_model.mna._branch_index('V%seq' % elt.name)
             relt = self.r_model.elements['R%seq' % elt.name]
-            v3_index = r_model._node_index(relt.nodenames[1])            
+            v3_index = r_model.mna._node_index(relt.nodenames[1])            
             
             if elt.is_inductor:
                 cls = Lcls
@@ -395,9 +374,9 @@ class Simulator(object):
             Zsubsdict[simcpt.Veqsym] = 0
 
         # Remove 1 / Req entries
-        Asym = r_model._A.subs(Asubsdict)
+        Asym = r_model.mna._A.subs(Asubsdict)
         # Remove Veq entries        
-        Zsym = r_model._Z.subs(Zsubsdict)
+        Zsym = r_model.mna._Z.subs(Zsubsdict)
 
         self.Asym = Asym
         self.Zsym = Zsym
@@ -408,8 +387,9 @@ class Simulator(object):
         # Convert to numpy ndarray
         self.A = array(Asym).astype(float)        
         
-        results = SimulationResults(tv, self.cct, r_model, r_model.node_list,
-                                    r_model.unknown_branch_currents)
+        results = SimulationResults(tv, self.cct, r_model,
+                                    r_model.node_list,
+                                    r_model.mna.unknown_branch_currents)
         
         for n, t1 in enumerate(tv):
             self._step(r_model, n, tv, results)
